@@ -1,4 +1,8 @@
 #![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]
+#![warn(rust_2018_idioms)]
+#![warn(clippy::all)]
+
+use std::{env, fs, panic, path::PathBuf, time::Instant};
 
 use glium::{
     glutin,
@@ -7,26 +11,39 @@ use glium::{
         event_loop::{ControlFlow, EventLoop, EventLoopProxy},
         window::WindowBuilder,
     },
-    {Display, Surface},
+    Display, Surface,
 };
-use image::Frame;
 use imgui::{Context, FontConfig, FontSource};
 use imgui_glium_renderer::Renderer;
 use imgui_winit_support::{HiDpiMode, WinitPlatform};
-use std::{env, panic, path::PathBuf, time::Instant};
+use serde_derive::{Deserialize, Serialize};
 
 mod app;
 mod clipboard;
 use app::App;
 mod background;
 use background::Background;
+mod icon;
 mod vec2;
 use vec2::Vec2;
-mod icon;
+mod rect;
+mod util;
+use util::UserEvent;
+mod image_io;
 
-pub enum UserEvent {
-    ImageLoaded(Option<Vec<Frame>>, Option<PathBuf>, Instant),
-    ImageError(String),
+#[derive(Serialize, Deserialize, Debug)]
+struct SaveData {
+    width: f64,
+    height: f64,
+}
+
+impl Default for SaveData {
+    fn default() -> Self {
+        Self {
+            width: 1100f64,
+            height: 720f64,
+        }
+    }
 }
 
 pub struct System {
@@ -43,6 +60,13 @@ pub struct System {
 
 impl System {
     pub fn new() -> Self {
+        #[cfg(target_os = "windows")]
+        {
+            native_windows_gui::enable_visual_styles();
+        }
+
+        let save_data = get_save_data();
+
         let event_loop: EventLoop<UserEvent> = EventLoop::with_user_event();
         let proxy = event_loop.create_proxy();
         let context = glutin::ContextBuilder::new().with_vsync(true);
@@ -50,12 +74,29 @@ impl System {
             .with_title(String::from("Simp"))
             .with_visible(false)
             .with_min_inner_size(glutin::dpi::LogicalSize::new(640f64, 400f64))
-            .with_inner_size(glutin::dpi::LogicalSize::new(1100f64, 720f64))
+            .with_inner_size(glutin::dpi::LogicalSize::new(
+                save_data.width,
+                save_data.height,
+            ))
             .with_window_icon(Some(icon::get_icon()));
+
         let display =
             Display::new(builder, context, &event_loop).expect("Failed to initialize display");
 
-        let app = App::new(proxy.clone(), [1100f32, 720f32]);
+        let app = {
+            let window_context = display.gl_window();
+            let window = window_context.window();
+
+            let pos = window.outer_position().unwrap();
+            let size = window.inner_size();
+
+            App::new(
+                proxy.clone(),
+                [size.width as f32, size.height as f32],
+                [pos.x, pos.y],
+                &display,
+            )
+        };
 
         let mut imgui = Context::create();
         imgui.set_ini_filename(None);
@@ -64,7 +105,7 @@ impl System {
         imgui.style_mut().anti_aliased_lines = false;*/
 
         if let Some(backend) = clipboard::init() {
-            imgui.set_clipboard_backend(Box::new(backend));
+            imgui.set_clipboard_backend(backend);
         } else {
             eprintln!("Failed to initialize clipboard");
         }
@@ -99,8 +140,15 @@ impl System {
         imgui.io_mut().font_global_scale = (1.0 / hidpi_factor) as f32;
 
         let renderer = Renderer::init(&mut imgui, &display).expect("Failed to initialize renderer");
+
         display.gl_window().window().set_visible(true);
 
+        let ctrl_proxy = proxy.clone();
+        ctrlc::set_handler(move || {
+            let _ = ctrl_proxy.send_event(UserEvent::Exit);
+        })
+        .unwrap();
+
         let background = Background::new(&display);
 
         Self {
@@ -158,11 +206,11 @@ impl System {
 
                     let gl_window = display.gl_window();
                     let mut target = display.draw();
-                    target.clear_color_srgb(0.262, 0.286, 0.337, 1.0);
+                    target.clear_color_srgb(0.156, 0.156, 0.156, 1.0);
 
                     let dimensions = display.get_framebuffer_dimensions();
                     let size = Vec2::new(dimensions.0 as f32, dimensions.1 as f32);
-                    background.render(&mut target, size);
+                    background.render(&mut target, size, app.top_bar_size);
 
                     if let Some(image) = app.image_view.as_mut() {
                         image.render(&mut target, size);
@@ -173,12 +221,22 @@ impl System {
                     renderer
                         .render(&mut target, draw_data)
                         .expect("Rendering failed");
+
+                    app.crop.render(&mut target, size);
+
                     target.finish().expect("Failed to swap buffers");
                 }
                 Event::WindowEvent {
                     event: WindowEvent::CloseRequested,
                     ..
                 } => *control_flow = ControlFlow::Exit,
+                Event::LoopDestroyed => {
+                    let data = SaveData {
+                        width: app.size.x() as f64,
+                        height: app.size.y() as f64,
+                    };
+                    save_data(&data);
+                }
                 mut event => {
                     {
                         let mut ui = imgui.frame();
@@ -212,11 +270,32 @@ impl System {
     }
 }
 
+fn get_save_data() -> SaveData {
+    if let Ok(data) = fs::read_to_string(get_data_path()) {
+        if let Ok(save) = ron::from_str::<SaveData>(&data) {
+            return save;
+        }
+    }
+    SaveData::default()
+}
+
+fn save_data(save_data: &SaveData) {
+    let data = ron::to_string(save_data).unwrap();
+    let _ = fs::write(get_data_path(), data);
+}
+
+fn get_data_path() -> PathBuf {
+    let dirs = directories::UserDirs::new().unwrap();
+    let mut home = dirs.home_dir().to_path_buf();
+    home.push(".simp.ron");
+    home
+}
+
 fn main() {
     panic::set_hook(Box::new(|panic_info| {
         let _ = msgbox::create(
             "Error",
-            &format!("panic occurred: {:?}", panic_info),
+            &format!("panic occurred: {}", panic_info),
             msgbox::IconType::Error,
         );
     }));
@@ -226,7 +305,12 @@ fn main() {
     let mut args: Vec<String> = env::args().collect();
     if args.len() > 1 {
         if let Some(arg) = args.pop() {
-            app::load_image::load_image(system.proxy.clone(), arg);
+            app::load_image::load(
+                system.proxy.clone(),
+                arg,
+                system.app.cache.clone(),
+                system.app.image_loader.clone(),
+            );
         }
     }
 
